// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _READINFO_H_
#define _READINFO_H_

#include <string>
#include <sstream>

class ReadInfo {
public:
	unsigned readNum;
	std::string readId;
	std::string readString;
	std::string reverseReadString;
	std::string readQuality;
	std::string reverseReadQuality;
	virtual ~ReadInfo() {
	}

	virtual unsigned getLength() {
		return readString.length();
	}

	void setReadNum(unsigned readNum) {
		this->readNum = readNum;
	}

	void setReadId(unsigned readId) {
		std::stringstream temp;
		temp << readId;
		this->readId = temp.str();
	}

	void setReadIdStr(const std::string& readId) {
		this->readId = readId;
	}

	void setReadString(const std::string& readString) {
		this->readString = readString;
	}

	void setReverseReadString(const std::string& reverseReadString) {
		this->reverseReadString = reverseReadString;
	}

	void setReadQuality(const std::string& readQuality) {
		this->readQuality = readQuality;
	}

	void setReverseReadQuality(const std::string& reverseReadQuality) {
		this->reverseReadQuality = reverseReadQuality;
	}

	void clear();
};

// TODO: There is no need to inherit any members of InputReadStructure. Okay for now.
// We should have PairedReadInfo and ReadInfo inherit from another abstract class.
// This will require casting to the appropriate subclass throughout the code.
// I chose this implementation to make sure no existing code breaks.
class PairedEndReadInfo: public ReadInfo {
public:
	ReadInfo first;
	ReadInfo second;
	virtual unsigned getLength() {
		return first.getLength();
	}
};

#endif
