// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _RESULTSTRINGWRITER_H_
#define _RESULTSTRINGWRITER_H_

#include "globals.h"
#include "search/searcher.h"
#include <vector>
#include <sstream>

enum SAMFlags {
	READ_UNMAPPED = 4,
	READ_REVERSE_MAPPED = 16,
	READ_PAIR_UNMAPPED1 = 77,
	READ_PAIR_UNMAPPED2 = 141,
	READ_PAIR_FORWARD_MAPPED1 = 99,
	READ_PAIR_REVERSE_MAPPED1 = 83,
	READ_PAIR_FORWARD_MAPPED2 = 147,
	READ_PAIR_REVERSE_MAPPED2 = 163,
	READ_PAIR_FIRST_FORWARD_MAPPED = 0,
	READ_PAIR_FIRST_REVERSE_MAPPED = 16,
	READ_PAIR_FIRST_UNMAPPED = 4,
	READ_PAIR_SECOND_FORWARD_MAPPED = 0,
	READ_PAIR_SECOND_REVERSE_MAPPED = 16,
	READ_PAIR_SECOND_UNMAPPED = 4,

	NOT_PRIMARY_ALIGN = 256
};

class ResultStringWriter {
public:
	HobbesConf& conf;

	ResultStringWriter(HobbesConf& conf) :
			conf(conf) {
	}

	virtual void createInitialString(std::stringstream& resultStream, Indexer& indexer) const {
	}

	virtual void writeResultString(std::stringstream& resultStream, ReadInfo& readInfo,
			Searcher* searcher) = 0;

protected:
	void getChromosomeOffset(std::vector<std::string>& chrNames, std::vector<unsigned>& chrLengths,
			unsigned absolutePos, std::string& chrName, unsigned& relativePos) const;
};

class SAMFormatResultStringWriter : public ResultStringWriter {
public:
	// TODO: change chars to appropriate data type when we start supporting those fields.
	// e.g. posNext to int/unsigned (see manual).

	// Unavailable.
	short unsigned mapQuality;
	// Unavailable.
	char refNext;
	// Unavailable.
	char posNext;
	// Unavailable.
	char templateLen;

	// #Optional fields - we support only the following 3 as of now. Note: first two flags are the same for us now
	// NM: Edit distance
	// XM: Substitution distance

	SAMFormatResultStringWriter(HobbesConf& conf) :
		ResultStringWriter(conf), mapQuality(UNAVAILABLE_3),
		refNext(UNAVAILABLE_1), posNext(UNAVAILABLE_2), templateLen(UNAVAILABLE_2) {
	}

	void createInitialString(std::stringstream& resultStream, Indexer& indexer) const;
	void writeResultString(std::stringstream& resultStream, ReadInfo& readInfo, Searcher* searcher);
};

// Paired.
class PairedSAMFormatResultStringWriter : public ResultStringWriter {
public:

	// TODO: Change chars to appropriate data type when we start supporting those fields.
	// e.g. posNext to int/unsigned (see manual)

	// Unavailable.
	short unsigned mapQuality;
	// Unavailable.
	char refNext;
	// Unavailable.
	char posNext;
	// Unavailable.
	char templateLen;

	// #Optional fields - we support only the following 3 as of now. Note: first two flags are the same for us now
	// NM: Edit distance
	// XM: Substitution distance
	// XA:Z: chr1,pos1,CIGAR1,NM1; chr2,pos2,CIGAR2,NM2; (other mappings)

	PairedSAMFormatResultStringWriter(HobbesConf& conf) :
		//ResultStringWriter(conf), samFlags(0), mapQuality(UNAVAILABLE_3),
		ResultStringWriter(conf), mapQuality(UNAVAILABLE_3),
		refNext(UNAVAILABLE_1), posNext(UNAVAILABLE_2), templateLen(UNAVAILABLE_2) {
	}

	void createInitialString(std::stringstream& resultStream, Indexer& indexer) const;
	void writeResultString(std::stringstream& resultStream, ReadInfo& readInfo, Searcher* searcher);
};
#endif
