#ifndef BITVECTOR_H_
#define BITVECTOR_H_

#include <iostream>
#include <stdlib.h>

using namespace std;

#define BITS_PER_BASE 3
#define BITS_LEFT_MASK       0x0FFFFFFFFFFFFFFF
#define BITS_SHIFT_LEFT_MASK 0xF000000000000000

class BitVector {

protected:
    unsigned long * U;     // 6 x 64, 64/3 = 21 characters each = 126 characters max
    int Length, seqLength;

public:

    BitVector()
    {
        Length = 0;
    }

	BitVector(int bp)
	{
		seqLength = bp;
		Length = seqLength/(sizeof(unsigned long)*8/BITS_PER_BASE);
		U = new (unsigned long[Length+1]);
	}
	
    BitVector(char * s)
    {
        SetSeq(s);
    }

    ~BitVector()
    {
        if (Length > 0) delete [] U;
    }

	void New(int bp)
	{
		if (Length > 0) delete [] U;

        int nsize = sizeof(unsigned long);
        int iter = (nsize*8)/BITS_PER_BASE;

        seqLength = bp;
        Length = (bp/iter);
        if (Length*iter < bp) Length++;
        U = new (unsigned long[Length+1]);
	}

    int Exp(int n, int e)
    {
        if (!e) return 1;

        int r = n;
        for(int j=1;j<e;j++)
            r *= n;
        return r;
    }

	void DumpRaw(char * msg)
	{
		cout << msg << " = ";
		for (int i=Length-1;i>=0;i--)
			cout << U[i] << " ";
		cout << endl;
	}
	
    void DumpToScreen(char * msg)
    {
        cout << msg << " = ";

        //for (int i=0; i<Length; i++)
        //    cout << "U[" << i << "] = " << U[i] << endl;

        char NA[7] = "-CGTAN";
        unsigned long * tU = new (unsigned long[Length+1]);
        unsigned long t;
        int index = 0;

        for (int i = 0; i < Length; i++)
        {
            //cout << "U[" << i << "] = " << U[i] << endl;
            tU[i] = U[i];
        }

        for (int i = 0; i < seqLength;i++)
        {
            t = U[0];
            index = (int)(t&0x0000000000000007);
            if (index < 6)
                cout << NA[index];
            else cout << "*";
            ShiftRight(BITS_PER_BASE);
        }
        cout << endl;

        for (int i = 0; i < Length; i++)
            U[i] = tU[i];
        if (Length > 0) delete [] tU;
    }

    int SetSeq(char * s)
    {
        if (Length > 0)
            delete [] U;

        int l = strlen(s);
        int nsize = sizeof(unsigned long);
        int iter = (nsize*8)/BITS_PER_BASE;

        seqLength = l;
        Length = (l/iter);
        if (Length*iter < l)
            Length++;

        U = new (unsigned long[Length+1]);
        for (int i=0; i<Length;i++)
            U[i] = 0;

        //cout << "INPUT ";
        for (int i=l-1; i >= 0; i--)
        {
            //cout << s[i];
            switch(s[i])
            {
                case 'a':
                case 'A':
                    U[0] += 4;
                    break;
                case 'c':
                case 'C':
                    U[0] += 1;
                    break;
                case 'g':
                case 'G':
                    U[0] += 2;
                    break;
                case 't':
                case 'T':
                    U[0] += 3;
                    break;
                case 'n':
                case 'N':
                    U[0] += 5;
                    break;
                default:
                    ShiftRight(BITS_PER_BASE);
                    return(-1);
            }
            ShiftLeft(BITS_PER_BASE);
        }
        //cout << endl;
        ShiftRight(BITS_PER_BASE);
        return 1;
    }

    unsigned char GetValue()
    {
        return (U[0]&0x0000000000000007);
    }

	void ShiftLeft(int nshift)
	{
	    for (int i=Length-1; i > 0;i--)
	    {
	        U[i] = (U[i] << nshift);
	        U[i] += (U[i-1] >> (64-nshift));
	    }
	    U[0] = (U[0] << nshift);
	}

	BitVector operator<<(int nshift)
	{
        BitVector rV;
        MakeCopy(rV);
	    for (int i=Length-1; i > 0;i--)
	    {
	        rV.U[i] = (U[i] << nshift);
	        rV.U[i] += (U[i-1] >> (64-nshift));
	    }
	    rV.U[0] = (U[0] << nshift);
	    return rV;
	}

	BitVector operator-(int v)
	{
        BitVector rV;
        MakeCopy(rV);
	    for (int i=0; i < Length;i++)
	    {
            if (rV.U[i] > 0)
            {
                rV.U[i] = U[i]-v;
                for (int n=i+1;n<Length;n++)
                    rV.U[n] = U[n];
                break;
            } rV.U[i] = U[i] - v;
	    }
	    return rV;
	}

    void ShiftRight(int nshift)
    {
	    for (int i=0; i < Length-1;i++)
	    {
	        U[i] = (U[i] >> nshift);
            U[i] += (U[i+1]<<(64-nshift));
	    }
	    U[Length-1] = (U[Length-1] >> nshift);
    }

    BitVector operator>>(int nshift)
    {
        BitVector rV;
        MakeCopy(rV);

	    for (int i=0; i < Length-1;i++)
	    {
	        rV.U[i] = (U[i] >> nshift);
            rV.U[i] += (U[i+1]<<(64-nshift));
	    }
	    rV.U[Length-1] = (U[Length-1] >> nshift);
	    return rV;
    }

    BitVector operator^(const BitVector & b)
    {
        BitVector rV;
        MakeCopy(rV);
        for (int i=0;i<Length;i++)
            rV.U[i] = U[i]^b.U[i];
        return rV;
    }

    BitVector operator~()
    {
        BitVector rV;
        MakeCopy(rV);
        for (int i=0;i<Length;i++)
            rV.U[i] = ~U[i];
        for (int i=0; i <Length;i++)
        {
			rV.U[i] += 1;
			if (rV.U[i] > ~U[i])
				break;
		}
        return rV;
    }
	
	BitVector & operator+=(int n)
	{
		U[0] += n;
		return * this;
	}
	
    BitVector operator|(const BitVector & b)
    {
        BitVector rV;
        MakeCopy(rV);
        for (int i=0;i<Length;i++)
            rV.U[i] = U[i]|b.U[i];
        return rV;
    }

    void MakeMask(int seqLen)
    {
        if (Length > 0) delete [] U;
        Length = (seqLen/21)+1;
        U = new (unsigned long[Length+1]);
        for (int i=0;i<Length;i++)
            U[i] = 0;
        for (int i = 0; i < seqLen; i++)
        {
            U[0] += 1;
            ShiftLeft(3);
        }
    }

    void MakeCopy(BitVector & b)
    {
        // garbage collection
        if (b.Length > 0)
            delete [] b.U;
        b.Length = Length;
        b.seqLength = seqLength;
        b.U = new (unsigned long[Length+1]);
    }

    BitVector operator&(const BitVector & b)
    {
        BitVector rV;
        MakeCopy(rV);
        for (int i=0;i<Length;i++)
            rV.U[i] = U[i]&b.U[i];
        return rV;
    }

    bool operator>(const BitVector & b)
    {
        for (int i=Length-1;i>=0;i--)
        {
            if (U[i]>b.U[i])
                return true;
            else if (U[i]<b.U[i])
                return false;
        }
        return false;
    }

    BitVector & operator=(const BitVector & b)
    {
        if (Length > 0)
            delete [] U;
        Length = b.Length;
        seqLength = b.seqLength;
        U = new (unsigned long[Length+1]);

        for (int i=0;i<Length;i++)
            U[i] = b.U[i];
        return * this;
    }

    BitVector & operator=(int n)
    {
        for (int i=0;i<Length;i++)
            U[i] = 0;
        U[0] = n;
        return * this;
    }

    BitVector & operator=(char * s)
    {
        SetSeq(s);
        return * this;
    }

	bool operator==(const BitVector & b)
	{
		return (U[0] == b.U[0]) && (U[1] == b.U[1]) && (U[2] == b.U[2])
				&& (U[3] == b.U[3]) && (U[4] == b.U[4]) && (U[5] == b.U[5]);
	}

	bool operator==(int n)
	{
        long m = 0;
        for (int i=0;i<Length;i++)
            m += U[i];

		return (m == n);
	}
};


#endif /* BITREAD_H_ */
