// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _PAIREDENDSEARCHER_H_
#define _PAIREDENDSEARCHER_H_

#include "search/searcher.h"
#include "search/pairedendmerger.h"

class PairedEndSearcher: public Searcher {
public:
	std::vector<unsigned> forwardCandidates1;
	std::vector<unsigned> forwardCandidates2;
	std::vector<PairedReadMapping> forwardResults;
	std::vector<unsigned> reverseCandidates1;
	std::vector<unsigned> reverseCandidates2;
	std::vector<PairedReadMapping> reverseResults;

	PairedEndRead forwardRead;
	PairedEndRead reverseRead;

	PairedEndSearcher(Indexer* indexer, unsigned distThresh, PairedEndMerger* merger,
			PairedEndVerifier* verifier, HobbesConf& conf, HobbesStats& stats) :
			Searcher(indexer, distThresh, conf, stats), merger(merger), verifier(verifier) {
		forwardCandidates1.reserve(CANDIS_PREALLOC_SIZE);
		forwardCandidates2.reserve(CANDIS_PREALLOC_SIZE);
		forwardResults.reserve(RESULTS_PREALLOC_SIZE);
		reverseCandidates1.reserve(CANDIS_PREALLOC_SIZE);
		reverseCandidates2.reserve(CANDIS_PREALLOC_SIZE);
		reverseResults.reserve(RESULTS_PREALLOC_SIZE);
	}

	virtual void findAllMappings(ReadInfo& readInfo);
	virtual void findKMappings(ReadInfo& readInfo);

	virtual MappingResultType getMappingResultType();

	virtual ~PairedEndSearcher() {
		delete merger;
	}

protected:
	PairedEndMerger* merger;
	PairedEndVerifier* verifier;

	virtual void prepare(unsigned readLen);
	void getStats(std::pair<unsigned, unsigned>& counts);
	void getBestStats();

	void mergeAndVerifyAll(PairedEndRead& read, std::vector<unsigned>& candidates1,
			std::vector<unsigned>& candidates2, std::vector<PairedReadMapping>& results);
	bool fillRead(PairedEndRead& read, unsigned& tOccurThresh);
	void genReverses(PairedEndReadInfo& readInfo);
	void setPairedRead(PairedEndReadInfo& readInfo);
};

class PairedEndHammingSearcher: public PairedEndSearcher {
public:
	PairedEndHammingSearcher(Indexer* indexer, unsigned distThresh, HobbesConf& conf,
			HobbesStats& stats) :
				PairedEndSearcher(
						indexer,
						distThresh,
						new PairedEndHammingMerger(indexer, distThresh),
						new PairedEndHammingVerifier(indexer, distThresh, conf.minInsertSize,
								conf.maxInsertSize, conf.queryType), conf, stats) {
	}
};

class PairedEndIndelSearcher: public PairedEndSearcher {
public:
	PairedEndIndelSearcher(Indexer* indexer, unsigned distThresh, HobbesConf& conf,
			HobbesStats& stats) :
				PairedEndSearcher(
						indexer,
						distThresh,
						new PairedEndIndelMerger(indexer, distThresh),
						new PairedEndIndelVerifier(indexer, distThresh, conf.minInsertSize, 
									conf.maxInsertSize, conf.queryType), conf, stats) {
	}
};

class PairedEndHeuristicIndelSearcher: public PairedEndSearcher {
public:
	PairedEndHeuristicIndelSearcher(Indexer* indexer, unsigned distThresh, HobbesConf& conf,
			HobbesStats& stats) :
				PairedEndSearcher(
						indexer,
						distThresh,
						new PairedEndHeuristicIndelMerger(indexer, distThresh),
						new PairedEndIndelVerifier(indexer, distThresh, conf.minInsertSize,
								conf.maxInsertSize, conf.queryType), conf, stats) {
	}
};

#endif
