// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#include "input/fasta.h"
#include "input/readinfo.h"

#include <algorithm>
#include <iostream>

using namespace std;

bool FastaInputReadQueue::nextRead() {
	ReadInfo* inputReadStruct = new ReadInfo();
	readStream.str(std::string());
	temp.clear();
	line.clear();
	if (getline(in, line)) {
		// Reading first line (read Id and optional desc)
		if (line[0] != '>') {
			cerr << "The file format is not FASTA. Please provide a FASTA file" << endl;
			delete inputReadStruct;
			return false;
		}
		temp = line.substr(1, line.size());
		inputReadStruct->setReadIdStr(temp);
		// Reading the "read" sequence until '+'
		while (getline(in, line)) {
			if (in.fail()) {
				cerr << "Error: Failure in reading stream" << endl;
				delete inputReadStruct;
				return false;
			}
			if (line[0] != '>') {
				readStream << line;
			} else {
				break;
			}
		}
		temp = readStream.str();
		transform(temp.begin(), temp.end(), temp.begin(), (int(*)(int))toupper);
		inputReadStruct->setReadString(temp);
	}
	if (in.fail()) {
		delete inputReadStruct;
		return false;
	}
	readsEnqueued++;
	inputReadStruct->setReadNum(readsEnqueued);
	inputReadStruct->setReadQuality(READ_QUAL_UNAVAIL);
	inputReadStruct->setReverseReadQuality(READ_QUAL_UNAVAIL);
	readQueue.push(inputReadStruct);
	return true;
}
